/* Install given floating-point control modes.  Alpha version.
   Copyright (C) 2016-2022 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */

#include <fenv_libc.h>

int
fesetmode (const femode_t *modep)
{
  unsigned long int fpcr;
  unsigned long int swcr;
  femode_t mode;

  /* As in fesetenv.  */
  if ((long int) modep >= 0)
    mode = *modep;
  else
    mode = (unsigned long int) modep;

  __asm__ __volatile__ ("excb; mf_fpcr %0" : "=f" (fpcr));
  fpcr = (fpcr & ~FPCR_ROUND_MASK) | (mode & FPCR_ROUND_MASK);
  __asm__ __volatile__ ("mt_fpcr %0" : : "f" (fpcr));

  swcr = __ieee_get_fp_control ();
  swcr = ((mode & SWCR_ALL_MASK & ~SWCR_STATUS_MASK)
	  | (swcr & SWCR_STATUS_MASK));
  __ieee_set_fp_control (swcr);

  return 0;
}
